@@ -33,7 +33,8 @@ from gettext import gettext as _
 from softwarecenter.enums import *
 
 from softwarecenter.view.appview import AppView, AppStore, AppViewFilter
-
+from softwarecenter.models.appstore import AppStore
+from softwarecenter.distro import get_distro
 from softwarecenter.view.softwarepane import SoftwarePane, wait_for_apt_cache_ready
 
 # TODO:
@@ -57,16 +58,14 @@ class OneConfPane(SoftwarePane):
                  oneconfeventhandler,
                  compared_with_hostid):
         # parent
-        SoftwarePane.__init__(self, cache, history, db, distro, icons, datadir)
+        SoftwarePane.__init__(self, cache, db, distro, icons, datadir)
         self.hostname = ''
+        self.pane_name = ''
         self.search_terms = ""
         self.compared_with_hostid = compared_with_hostid
         self.current_appview_selection = None
         self.apps_filter = None
-        try:
-            self.nonapps_visible = AppStore.NONAPPS_NEVER_VISIBLE
-        except AttributeError: # be compatible with older USC
-            self.nonapps_visible = False
+        self.nonapps_visible = AppStore.NONAPPS_NEVER_VISIBLE
         self.refreshing = False
 
         # OneConf stuff there
@@ -76,23 +75,29 @@ class OneConfPane(SoftwarePane):
         # Backend installation
         self.backend.connect("transaction-finished", self._on_transaction_finished)
 
-        # UI
-        self._build_ui()
+    def init_view(self):
+        if not self.view_initialized:
+            self.show_appview_spinner()
+            super(OneConfPane, self).init_view()
+            self._build_ui()
+            self.view_initialized = True
+            self.force_refresh()
 
     def _build_ui(self):
 
         self.navigation_bar.set_size_request(26, -1)
-        self.notebook.append_page(self.scroll_app_list, gtk.Label("app list"))
+        self.notebook.append_page(self.box_app_list, gtk.Label(NAV_BUTTON_ID_LIST))
         # details
-        self.notebook.append_page(self.scroll_details, gtk.Label("details"))
+        self.notebook.append_page(self.scroll_details, gtk.Label(NAV_BUTTON_ID_DETAILS))
+        self.scroll_details.show()
 
         self.embeeded_title_bar = gtk.HBox()
         self.toolbar = gtk.Toolbar()
         self.toolbar.show()
         self.toolbar.set_style(gtk.TOOLBAR_TEXT)
-        self.pack_start(self.embeeded_title_bar, expand=False)
+        self.box_app_list.pack_start(self.embeeded_title_bar, expand=False)
         self.embeeded_title_bar.pack_start(self.toolbar, expand=True)
-        self.reorder_child(self.embeeded_title_bar, 1)
+        self.box_app_list.reorder_child(self.embeeded_title_bar, 1)
 
         additional_pkg_action = gtk.RadioAction('additional_pkg', None, None, None, self.ADDITIONAL_PKG)
         additional_pkg_action.connect('changed', self.change_current_mode)
@@ -109,9 +114,8 @@ class OneConfPane(SoftwarePane):
         self.embeeded_title_bar.pack_end(self.act_on_store_button, expand=False)
         self.act_on_store_button.connect('clicked', self._act_on_current_appstore)
         self.act_on_store_button.show()
-
-        # initial refresh
-        self.force_refresh()
+        
+        self.box_app_list.show_all()
 
     def _act_on_current_appstore(self, widget):
         '''
@@ -142,6 +146,10 @@ class OneConfPane(SoftwarePane):
             gobject.timeout_add_seconds(time, Thread(target=self._on_inventory_change, args=(self.oneconfeventhandler,)).start)
 
     def _on_inventory_change(self, oneconfeventhandler):
+        
+        # only make oneconf calls once initialized
+        if not self.view_initialized:
+            return
         try:
             current, hostname, show_inventory, show_others = oneconfeventhandler.u1hosts[self.compared_with_hostid]
         except KeyError:
@@ -149,8 +157,9 @@ class OneConfPane(SoftwarePane):
             return
         # create first filter
         if not self.apps_filter:
-            self.apps_filter = OneConfFilter(self.db, self.cache, set(), set())
+            self.apps_filter = OneConfFilter(self.db, self.cache, set(), set(), self.nonapps_visible)
         self.hostname = hostname
+        self.pane_name = hostname
         (additional_pkg, missing_pkg) = oneconfeventhandler.oneconf.diff_selection(self.compared_with_hostid, '', True)
         self.apps_filter.additional_pkglist = set(additional_pkg)
         self.apps_filter.removed_pkglist = set(missing_pkg)
@@ -161,21 +170,16 @@ class OneConfPane(SoftwarePane):
         gobject.timeout_add(1, self.refresh_apps)
 
     def refresh_selection_bar(self):
-        try: # be compatible with older USC
-            if self.nonapps_visible == AppStore.NONAPPS_ALWAYS_VISIBLE:
-                nonapps_visible = True
-            else:
-                nonapps_visible = False
-        except AttributeError:
-            nonapps_visible = self.nonapps_visible
-        if nonapps_visible:
-            number_additional_pkg = len(self.apps_filter.additional_pkg)
-            number_removed_pkg = len(self.apps_filter.removed_pkg)
+        if self.nonapps_visible == AppStore.NONAPPS_ALWAYS_VISIBLE:
+            number_additional_pkg = len(self.apps_filter.additional_apps_pkg)
+            number_removed_pkg = len(self.apps_filter.removed_apps_pkg)
         else:
-            number_additional_pkg = self.apps_filter.additional_apps_pkg
-            number_removed_pkg = self.apps_filter.removed_apps_pkg
+            number_additional_pkg = len(self.apps_filter.additional_apps_pkg)
+            number_removed_pkg = len(self.apps_filter.removed_apps_pkg)
+
+        # FIXME: use positive language, use ngettext
         if number_additional_pkg > 1:
-            msg_additional_pkg = _('%s items that aren\'t on that computer') % number_additional_pkg
+            msg_additional_pkg = _('%s new items that are on the remove computer') % number_additional_pkg
             msg_add_act_on_store = _("Install those %s items") % number_additional_pkg
         else:
             msg_additional_pkg = _('%s item that isn\'t on that computer') % number_additional_pkg
@@ -211,7 +215,7 @@ class OneConfPane(SoftwarePane):
 
     def _show_installed_overview(self):
         " helper that goes back to the overview page "
-        self.navigation_bar.remove_id("details")
+        self.navigation_bar.remove_id(NAV_BUTTON_ID_DETAILS)
         self.notebook.set_current_page(self.PAGE_APPLIST)
         self.toolbar.show()
         self.searchentry.show()
@@ -219,7 +223,8 @@ class OneConfPane(SoftwarePane):
     def _clear_search(self):
         # remove the details and clear the search
         self.searchentry.clear()
-        self.navigation_bar.remove_id("search")
+        self.apps_search_term = ""
+        self.navigation_bar.remove_id(NAV_BUTTON_ID_SEARCH)
 
     @wait_for_apt_cache_ready
     def refresh_apps(self):
@@ -231,46 +236,44 @@ class OneConfPane(SoftwarePane):
         if self.refreshing:
             return False
         self.refreshing = True
-
         self.apps_filter.reset_counter()
-        if self.search_terms:
-            query = self.db.get_query_list_from_search_entry(self.search_terms)
-            self.navigation_bar.add_with_id(_("Search Results"),
-                                            self.on_navigation_search, 
-                                            "search")
-        else:
-            query = None
-        self.navigation_bar.add_with_id(self.hostname, 
-                                        self.on_navigation_list,
-                                        "list",
-                                        animate=False)
-        # *ugh* deactivate the old model because otherwise it keeps
-        # getting progress_changed events and eats CPU time until it's
-        # garbage collected
-        old_model = self.app_view.get_model()
-        if old_model is not None:
-            old_model.active = False
-        # get a new store and attach it to the view
-        new_model = AppStore(self.cache,
-                             self.db, 
-                             self.icons, 
-                             query,
-                             nonapps_visible = self.nonapps_visible,
-                             filter=self.apps_filter)
-        self.app_view.set_model(new_model)
-        self.emit("app-list-changed", len(new_model))
+
+        # call parent to do the real work
+        super(OneConfPane, self).refresh_apps()
+        
+        # FIXME: his is fake just to see if the label shows up
+        #self.app_view.get_model().nr_apps = 1
+        #self.app_view.get_model().nr_pkgs = 9
+
+        self.update_show_hide_nonapps()
         self.refresh_selection_bar()
         self.refreshing = False
         return False
 
-    def on_search_terms_changed(self, searchentry, terms):
+    def is_applist_view_showing(self):
+        # FIXME: actually make this useful
+        return True
+        
+    def on_search_terms_changed(self, widget, new_text):
         """callback when the search entry widget changes"""
-        logging.debug("on_search_terms_changed: '%s'" % terms)
-        self.search_terms = terms
-        if not self.search_terms:
+        logging.debug("on_search_terms_changed: %s" % new_text)
+
+        # we got the signal after we already switched to a details
+        # page, ignore it
+        if self.notebook.get_current_page() == self.PAGE_APP_DETAILS:
+            return
+
+        # DTRT if the search is reseted
+        if not new_text:
             self._clear_search()
+        else:
+            self.apps_search_term = new_text
+            self.apps_limit = DEFAULT_SEARCH_LIMIT
+            # enter custom list mode if search has non-trailing
+            # comma per custom list spec.
+            self.custom_list_mode = "," in new_text.rstrip(',')
         self._append_refresh_apps()
-        self.notebook.set_current_page(self.PAGE_APPLIST)
+        self.notebook.set_current_page(self.PAGE_APPLIST)        
 
     def on_db_reopen(self, db):
         self._append_refresh_apps()
@@ -305,7 +308,7 @@ class OneConfPane(SoftwarePane):
         self.current_appview_selection = app
 
     def display_search(self):
-        self.navigation_bar.remove_id("details")
+        self.navigation_bar.remove_id(NAV_BUTTON_ID_DETAILS)
         self.notebook.set_current_page(self.PAGE_APPLIST)
         model = self.app_view.get_model()
         if model:
@@ -339,45 +342,83 @@ class OneConfPane(SoftwarePane):
     def is_category_view_showing(self):
         # there is no category view in the OneConf pane
         return False
+        
+    def _hide_nonapp_pkgs(self):
+        # override to never show apps visible
+        self.nonapps_visible = AppStore.NONAPPS_NEVER_VISIBLE
+        self.apps_filter.set_non_apps_visible(self.nonapps_visible)
+        self.refresh_apps()
 
+    def _show_nonapp_pkgs(self):
+        # override to never show apps visible
+        self.nonapps_visible = AppStore.NONAPPS_ALWAYS_VISIBLE
+        self.apps_filter.set_non_apps_visible(self.nonapps_visible)
+        self.refresh_apps()
 
 # TODO: find a way to replace that by a Xapian query ?
-class OneConfFilter(object):
+class OneConfFilter(xapian.MatchDecider):
     """
-    Filter that can be hooked into AppStore to filter for pkg name criteria
+    Filter that can be hooked into xapian get_mset to filter for criteria that
+    are based around the package details that are not listed in xapian
+    (like installed_only) or archive section
     """
+    
     (ADDITIONAL_PKG, REMOVED_PKG) = range(2)
 
-    def __init__(self, db, cache, additional_pkglist, removed_pkglist):
+    def __init__(self, db, cache, additional_pkglist, removed_pkglist, non_apps_visible):
+        xapian.MatchDecider.__init__(self)
+        self.distro = get_distro()
         self.db = db
         self.cache = cache
+        self.supported_only = False
+        self.installed_only = False
+        self.not_installed_only = False
         self.additional_pkglist = additional_pkglist
         self.removed_pkglist = removed_pkglist
-        self.only_packages_without_applications = False
         self.current_mode = self.ADDITIONAL_PKG
+        self._non_apps_visible = non_apps_visible
         self.reset_counter()
-    def set_only_packages_without_applications(self, v):
-        """
-        only show packages that are not displayed as applications
-
-        e.g. abiword (the package document) will not be displayed
-             because there is a abiword application already
-        """
-        self.only_packages_without_applications = v
-    def get_only_packages_without_applications(self, v):
-        return self.only_packages_without_applications
+    @property
+    def required(self):
+        """ True if the filter is in a state that it should be part of a query """
+        return True
     def set_new_current_mode(self, v):
         self.current_mode = v
     def get_current_mode(self):
         return self.current_mode
     def get_supported_only(self):
+        return self.supported_only
+    def set_non_apps_visible(self, non_apps_visible):
+        self._non_apps_visible = non_apps_visible
+    def __eq__(self, other):
+        #if self is None and other is not None: 
+        #    return True
+        #if self is None or other is None: 
+        #    return False
+        #return (self.current_mode == other.current_mode and
+        #        self.additional_pkglist == other.additional_pkglist and
+        #        self.removed_pkglist == other.removed_pkglist)
+        # FIXME: EVILHACK
+        # let be evil for now and reforce all the reference. This bad hack can be removed once:
+        # 1. __call__ doesn't need the filtering to be performed again for counting the number of package both side (so a one-only tree will be needed)
+        # 2. we create a new signal other than "inventory-refreshed" to first check if the list of host is different or if something really happened
+        # Note: evil here mean "as in maverick" case as there was not this kind of filter :)
         return False
+    def __ne__(self, other):
+        return not self.__eq__(other)
     def reset_counter(self):
-        self.additional_apps_pkg = 0
-        self.removed_apps_pkg = 0
-    def filter(self, doc, pkgname):
+        self.additional_apps_pkg = set()
+        self.removed_apps_pkg = set()
+    def _current_package_is_app(self, pkgname):
+        if not self._package_cache_is_app:
+            self._package_name_is_app = (self._non_apps_visible == AppStore.NONAPPS_ALWAYS_VISIBLE or
+                                         len(self.db.get_apps_for_pkgname(pkgname)) == 1)
+        return self._package_name_is_app
+    def __call__(self, doc):
         """return True if the package should be displayed"""
-
+        pkgname =  self.db.get_pkgname(doc)
+        self._package_cache_is_app = None
+        
         if self.current_mode == self.ADDITIONAL_PKG:
             pkg_list_to_compare = self.additional_pkglist
             other_list = self.removed_pkglist
@@ -389,17 +430,18 @@ class OneConfFilter(object):
         # for that (OneConf doesn't know which packages are applications)
         # it would be better.
 
-        if pkgname in other_list:
+        if pkgname in other_list and self._current_package_is_app(pkgname): 
             if self.current_mode == self.ADDITIONAL_PKG:
-                self.removed_apps_pkg += 1
+                self.removed_apps_pkg.add(pkgname)
             else:
-                self.additional_apps_pkg += 1
+                self.additional_apps_pkg.add(pkgname)
                                     
         if pkgname in pkg_list_to_compare:
-            if self.current_mode == self.ADDITIONAL_PKG:
-                self.additional_apps_pkg += 1
-            else:
-                self.removed_apps_pkg += 1
+            if self._current_package_is_app(pkgname):
+                if self.current_mode == self.ADDITIONAL_PKG:
+                    self.additional_apps_pkg.add(pkgname)
+                else:
+                    self.removed_apps_pkg.add(pkgname)
             return True
         return False
 
